@@ -113,7 +113,7 @@ static int create_hashes(struct globals *globals)
 	return 0;
 }
 
-int set_best_server(struct globals *globals)
+static int set_best_server(struct globals *globals)
 {
 	struct hash_it_t *hashit = NULL;
 	struct server *best_server = NULL;
@@ -218,6 +218,59 @@ static int purge_data(struct globals *globals)
 	return 0;
 }
 
+static void update_server_info(struct globals *globals)
+{
+	struct hash_it_t *hashit = NULL;
+	struct interface *interface;
+	struct ether_addr *macaddr;
+	struct hashtable_t *tg_hash = NULL;
+	struct hashtable_t *orig_hash = NULL;
+
+	/* TQ is not used for master sync mode */
+	if (globals->opmode == OPMODE_MASTER)
+		return;
+
+	if (strcmp(globals->mesh_iface, "none") != 0) {
+		tg_hash = tg_hash_new(globals->mesh_iface);
+		if (!tg_hash) {
+			fprintf(stderr, "Failed to create translation hash\n");
+			return;
+		}
+
+		orig_hash = orig_hash_new(globals->mesh_iface);
+		if (!orig_hash) {
+			fprintf(stderr, "Failed to create originator hash\n");
+			goto free_tg_hash;
+		}
+	}
+
+	list_for_each_entry(interface, &globals->interfaces, list) {
+		while (NULL != (hashit = hash_iterate(interface->server_hash,
+						      hashit))) {
+			struct server *server = hashit->bucket->data;
+
+			if (!orig_hash) {
+				server->tq = 255;
+				continue;
+			}
+
+			macaddr = translate_mac(tg_hash, &server->hwaddr);
+			if (macaddr)
+				server->tq = get_tq(orig_hash, macaddr);
+			else
+				server->tq = 0;
+		}
+	}
+
+	set_best_server(globals);
+
+	if (orig_hash)
+		orig_hash_free(orig_hash);
+free_tg_hash:
+	if (tg_hash)
+		tg_hash_free(tg_hash);
+}
+
 static void check_if_socket(struct interface *interface, struct globals *globals)
 {
 	int sock;
@@ -422,6 +475,7 @@ int alfred_server(struct globals *globals)
 			sync_data(globals);
 		} else {
 			/* send local data to server */
+			update_server_info(globals);
 			push_local_data(globals);
 		}
 		purge_data(globals);
